@@ -11,16 +11,17 @@
 
 """# Taipy Core
 
-The Taipy Core package is a Python library designed to build powerful and customized data-driven back-end
-applications. It provides the necessary tools to help Python developers transform their algorithms into a
+The Taipy Core package is a Python library designed to build powerful, customized, data-driven back-end
+applications. It provides the tools to help Python developers transform their algorithms into a
 complete back-end application.
 More details on the [Taipy Core](../../../manuals/core/) functionalities are available in the user manual.
 
-To build a Taipy Core application, the first step consists of setting up the Taipy configuration to design the
-characteristics and the behavior of your application.
+To build a Taipy Core application, the first step consists of setting up the Taipy configuration to design
+your application's characteristics and behaviors.
 Import `Config^` from the `taipy.config^` module, then use the various methods of the `Config^` singleton class to
-configure your core application. In particular configure the [data nodes](../../../manuals/core/config/data-node-config,
-[tasks](../../../manuals/core/config/task-config), and [scenarios](../../../manuals/core/config/scenario-config).
+configure your core application. In particular, configure the
+[data nodes](../../../manuals/core/config/data-node-config), [tasks](../../../manuals/core/config/task-config),
+and [scenarios](../../../manuals/core/config/scenario-config).
 Please refer to the [Core configuration user manual](../../../manuals/core/config/) for more information and
 detailed examples.
 
@@ -30,10 +31,10 @@ in the following [function](./#functions) section. In particular, the most used
 
 !!! Note
 
-    Taipy Core provides a runnable service `Core^`. can run as a service in a dedicated thread. The purpose is to
+    Taipy Core provides a runnable service, `Core^` that runs as a service in a dedicated thread. The purpose is to
     have a dedicated thread responsible for dispatching the submitted jobs to an available executor for their execution.
 
-    In particular, this `Core^` service is automatically run when Core is used with taipy-rest or taipy-gui. See
+    In particular, this `Core^` service is automatically run when Core is used with taipy-rest or taipy-gui. See the
     [running services](../../../manuals/running_services/) page of the user manual for more details.
 
 """
@@ -41,7 +42,7 @@ import json
 import os
 
 from ._core import Core
-from .common.alias import CycleId, DataNodeId, JobId, PipelineId, ScenarioId, TaskId
+from .common.alias import CycleId, DataNodeId, Edit, JobId, PipelineId, ScenarioId, TaskId
 from .common.default_custom_document import DefaultCustomDocument
 from .cycle.cycle import Cycle
 from .data.data_node import DataNode
