@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -12,14 +12,16 @@
 import csv
 from datetime import datetime, timedelta
 from os.path import isfile
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Set
 
+import modin.pandas as modin_pd
 import pandas as pd
 
 from taipy.config.common.scope import Scope
 
+from .._version._version_manager_factory import _VersionManagerFactory
 from ..common._reload import _self_reload
-from ..common.alias import DataNodeId, JobId
+from ..common.alias import DataNodeId, Edit, JobId
 from ..exceptions.exceptions import InvalidExposedType, MissingRequiredProperty
 from .data_node import DataNode
 
@@ -33,25 +35,32 @@ class CSVDataNode(DataNode):
         scope (Scope^): The scope of this data node.
         id (str): The unique identifier of this data node.
         name (str): A user-readable name of this data node.
-        parent_id (str): The identifier of the parent (pipeline_id, scenario_id, cycle_id) or `None`.
+        owner_id (str): The identifier of the owner (pipeline_id, scenario_id, cycle_id) or `None`.
+        parent_ids (Optional[Set[str]]): The identifiers of the parent tasks or `None`.
         last_edit_date (datetime): The date and time of the last modification.
-        job_ids (List[str]): The ordered list of jobs that have written this data node.
-        cacheable (bool): True if this data node is cacheable. False otherwise.
-        validity_period (Optional[timedelta]): The validity period of a cacheable data node.
+        edits (List[Edit^]): The ordered list of edits for that job.
+        version (str): The string indicates the application version of the data node to instantiate. If not provided,
+            the current version is used.
+        validity_period (Optional[timedelta]): The validity period of a data node.
             Implemented as a timedelta. If _validity_period_ is set to None, the data_node is
             always up-to-date.
         edit_in_progress (bool): True if a task computing the data node has been submitted
             and not completed yet. False otherwise.
         path (str): The path to the CSV file.
         properties (dict[str, Any]): A dictionary of additional properties. The _properties_
-            must have a _"default_path"_ or _"path"_ entry with the path of the CSV file.
+            must have a _"default_path"_ or _"path"_ entry with the path of the CSV file:
+
+            - _"default_path"_ `(str)`: The default path of the CSV file.\n
+            - _"has_header"_ `(bool)`: If True, indicates that the CSV file has a header.\n
+            - _"exposed_type"_: The exposed type of the data read from CSV file. The default value is `pandas`.\n
     """
 
     __STORAGE_TYPE = "csv"
     __EXPOSED_TYPE_PROPERTY = "exposed_type"
     __EXPOSED_TYPE_NUMPY = "numpy"
     __EXPOSED_TYPE_PANDAS = "pandas"
-    __VALID_STRING_EXPOSED_TYPES = [__EXPOSED_TYPE_PANDAS, __EXPOSED_TYPE_NUMPY]
+    __EXPOSED_TYPE_MODIN = "modin"
+    __VALID_STRING_EXPOSED_TYPES = [__EXPOSED_TYPE_PANDAS, __EXPOSED_TYPE_MODIN, __EXPOSED_TYPE_NUMPY]
     __PATH_KEY = "path"
     __DEFAULT_PATH_KEY = "default_path"
     __HAS_HEADER_PROPERTY = "has_header"
@@ -63,10 +72,11 @@ class CSVDataNode(DataNode):
         scope: Scope,
         id: Optional[DataNodeId] = None,
         name: Optional[str] = None,
-        parent_id: Optional[str] = None,
+        owner_id: Optional[str] = None,
+        parent_ids: Optional[Set[str]] = None,
         last_edit_date: Optional[datetime] = None,
-        job_ids: List[JobId] = None,
-        cacheable: bool = False,
+        edits: List[Edit] = None,
+        version: str = None,
         validity_period: Optional[timedelta] = None,
         edit_in_progress: bool = False,
         properties: Dict = None,
@@ -96,10 +106,11 @@ class CSVDataNode(DataNode):
             scope,
             id,
             name,
-            parent_id,
+            owner_id,
+            parent_ids,
             last_edit_date,
-            job_ids,
-            cacheable,
+            edits,
+            version or _VersionManagerFactory._build_manager()._get_latest_version(),
             validity_period,
             edit_in_progress,
             **properties,
@@ -124,12 +135,15 @@ class CSVDataNode(DataNode):
     def _check_exposed_type(self, exposed_type):
         if isinstance(exposed_type, str) and exposed_type not in self.__VALID_STRING_EXPOSED_TYPES:
             raise InvalidExposedType(
-                f"Invalid string exposed type {exposed_type}. Supported values are {', '.join(self.__VALID_STRING_EXPOSED_TYPES)}"
+                f"Invalid string exposed type {exposed_type}. Supported values are "
+                f"{', '.join(self.__VALID_STRING_EXPOSED_TYPES)}"
             )
 
     def _read(self):
         if self.properties[self.__EXPOSED_TYPE_PROPERTY] == self.__EXPOSED_TYPE_PANDAS:
             return self._read_as_pandas_dataframe()
+        if self.properties[self.__EXPOSED_TYPE_PROPERTY] == self.__EXPOSED_TYPE_MODIN:
+            return self._read_as_modin_dataframe()
         if self.properties[self.__EXPOSED_TYPE_PROPERTY] == self.__EXPOSED_TYPE_NUMPY:
             return self._read_as_numpy()
         return self._read_as()
@@ -153,7 +167,9 @@ class CSVDataNode(DataNode):
     def _read_as_numpy(self):
         return self._read_as_pandas_dataframe().to_numpy()
 
-    def _read_as_pandas_dataframe(self, usecols: Optional[List[int]] = None, column_names: Optional[List[str]] = None):
+    def _read_as_pandas_dataframe(
+        self, usecols: Optional[List[int]] = None, column_names: Optional[List[str]] = None
+    ) -> pd.DataFrame:
         try:
             if self.properties[self.__HAS_HEADER_PROPERTY]:
                 if column_names:
@@ -166,8 +182,26 @@ class CSVDataNode(DataNode):
         except pd.errors.EmptyDataError:
             return pd.DataFrame()
 
+    def _read_as_modin_dataframe(
+        self, usecols: Optional[List[int]] = None, column_names: Optional[List[str]] = None
+    ) -> modin_pd.DataFrame:
+        try:
+            if self.properties[self.__HAS_HEADER_PROPERTY]:
+                if column_names:
+                    return modin_pd.read_csv(self._path)[column_names]
+                return modin_pd.read_csv(self._path)
+            else:
+                if usecols:
+                    return modin_pd.read_csv(self._path, header=None, usecols=usecols)
+                return modin_pd.read_csv(self._path, header=None)
+        except pd.errors.EmptyDataError:
+            return modin_pd.DataFrame()
+
     def _write(self, data: Any):
-        pd.DataFrame(data).to_csv(self._path, index=False)
+        if isinstance(data, (pd.DataFrame, modin_pd.DataFrame)):
+            data.to_csv(self._path, index=False)
+        else:
+            pd.DataFrame(data).to_csv(self._path, index=False)
 
     def write_with_column_names(self, data: Any, columns: List[str] = None, job_id: Optional[JobId] = None):
         """Write a selection of columns.
@@ -182,6 +216,4 @@ class CSVDataNode(DataNode):
         else:
             df = pd.DataFrame(data, columns=columns)
         df.to_csv(self._path, index=False)
-        self._last_edit_date = datetime.now()
-        if job_id:
-            self.job_ids.append(job_id)
+        self._track_edit(timestamp=datetime.now(), job_id=job_id)
