@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -9,12 +9,14 @@
 # an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 # specific language governing permissions and limitations under the License.
 
+import os
 import uuid
 from abc import abstractmethod
 from datetime import datetime, timedelta
 from functools import reduce
-from typing import Any, List, Optional, Tuple, Union
+from typing import Any, List, Optional, Set, Tuple, Union
 
+import modin.pandas as modin_pd
 import numpy as np
 import pandas as pd
 
@@ -22,12 +24,12 @@ from taipy.config.common._validate_id import _validate_id
 from taipy.config.common.scope import Scope
 from taipy.logger._taipy_logger import _TaipyLogger
 
+from .._version._version_manager_factory import _VersionManagerFactory
 from ..common._entity import _Entity
-from ..common._listattributes import _ListAttributes
 from ..common._properties import _Properties
 from ..common._reload import _reload, _self_reload, _self_setter
 from ..common._warnings import _warn_deprecated
-from ..common.alias import DataNodeId, JobId
+from ..common.alias import DataNodeId, Edit, JobId
 from ..exceptions.exceptions import NoData
 from ._filter import _FilterDataNode
 from .operator import JoinOperator, Operator
@@ -37,7 +39,7 @@ class DataNode(_Entity):
     """Reference to a dataset.
 
     A Data Node is an abstract class that holds metadata related to the dataset it refers to.
-    In particular, a data node holds the name, the scope, the parent identifier, the last
+    In particular, a data node holds the name, the scope, the owner identifier, the last
     edit date, and some additional properties of the data.<br/>
     A Data Node also contains information and methods needed to access the dataset. This
     information depends on the type of storage, and it is held by subclasses (such as
@@ -52,16 +54,17 @@ class DataNode(_Entity):
         scope (Scope^): The scope of this data node.
         id (str): The unique identifier of this data node.
         name (str): A user-readable name of this data node.
-        parent_id (str): The identifier of the parent (pipeline_id, scenario_id, cycle_id) or
-            None.
+        owner_id (str): The identifier of the owner (pipeline_id, scenario_id, cycle_id) or
+            `None`.
+        parent_ids (Optional[Set[str]]): The set of identifiers of the parent tasks.
         last_edit_date (datetime): The date and time of the last modification.
-        job_ids (List[str]): The ordered list of jobs that have written this data node.
-        cacheable (bool): True if this data node is cacheable. False otherwise.
-        validity_period (Optional[timedelta]): The validity period of a cacheable data node.
+        edits (List[Edit^]): The list of Edits (an alias for dict) containing medata about each edition of that node.
+        version (str): The string indicates the application version of the data node to instantiate. If not provided,
+            the current version is used.
+        validity_period (Optional[timedelta]): The validity period of a data node.
             Implemented as a timedelta. If _validity_period_ is set to None, the data_node is
             always up-to-date.
-        edit_in_progress (bool): True if a task computing the data node has been submitted
-            and not completed yet. False otherwise.
+        edit_in_progress (bool): True if the data node is locked for modification. False otherwise.
         kwargs: A dictionary of additional properties.
     """
 
@@ -70,6 +73,7 @@ class DataNode(_Entity):
     __logger = _TaipyLogger._get_logger()
     _REQUIRED_PROPERTIES: List[str] = []
     _MANAGER_NAME = "data"
+    __PATH_KEY = "path"
 
     def __init__(
         self,
@@ -77,32 +81,79 @@ class DataNode(_Entity):
         scope: Scope = Scope(Scope.PIPELINE),
         id: Optional[DataNodeId] = None,
         name: Optional[str] = None,
-        parent_id: Optional[str] = None,
+        owner_id: Optional[str] = None,
+        parent_ids: Optional[Set[str]] = None,
         last_edit_date: Optional[datetime] = None,
-        job_ids: List[JobId] = None,
-        cacheable: bool = False,
+        edits: List[Edit] = None,
+        version: str = None,
         validity_period: Optional[timedelta] = None,
         edit_in_progress: bool = False,
         **kwargs,
     ):
         self.config_id = _validate_id(config_id)
         self.id = id or DataNodeId(self.__ID_SEPARATOR.join([self._ID_PREFIX, self.config_id, str(uuid.uuid4())]))
-        self.parent_id = parent_id
+        self.owner_id = owner_id
+        self._parent_ids = parent_ids or set()
         self._scope = scope
         self._last_edit_date = last_edit_date
         self._name = name or self.id
         self._edit_in_progress = edit_in_progress
-        self._job_ids = _ListAttributes(self, job_ids or list())
-
-        self._cacheable = cacheable
+        self._version = version or _VersionManagerFactory._build_manager()._get_latest_version()
         self._validity_period = validity_period
 
+        # Track edits
+        self._edits = edits or list()
+
         self._properties = _Properties(self, **kwargs)
 
+    @property  # type: ignore
+    def parent_id(self):
+        """
+        Deprecated. Use owner_id instead.
+        """
+        _warn_deprecated("parent_id", suggest="owner_id")
+        return self.owner_id
+
+    @parent_id.setter  # type: ignore
+    def parent_id(self, val):
+        """
+        Deprecated. Use owner_id instead.
+        """
+        _warn_deprecated("parent_id", suggest="owner_id")
+        self.owner_id = val
+
+    def get_parents(self):
+        """Get all parents of the data node."""
+        from ... import core as tp
+
+        return tp.get_parents(self)
+
+    @property  # type: ignore
+    @_self_reload(_MANAGER_NAME)
+    def parent_ids(self):
+        """List of parent ids of the data node."""
+        return self._parent_ids
+
+    @property  # type: ignore
+    @_self_reload(_MANAGER_NAME)
+    def edits(self):
+        """Get all `Edit^`s of the data node."""
+        return self._edits
+
+    def get_last_edit(self):
+        """Get last `Edit^` of the data node, or None"""
+        if self._edits:
+            return self._edits[-1]
+        return None
+
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
     def last_edit_date(self):
-        return self._last_edit_date
+        last_modified_datetime = self.__get_last_modified_datetime()
+        if last_modified_datetime and last_modified_datetime > self._last_edit_date:
+            return last_modified_datetime
+        else:
+            return self._last_edit_date
 
     @last_edit_date.setter  # type: ignore
     @_self_setter(_MANAGER_NAME)
@@ -111,9 +162,7 @@ class DataNode(_Entity):
 
     @property  # type: ignore
     def last_edition_date(self):
-        """
-        Deprecated. Use last_edit_date instead.
-        """
+        """Deprecated. Use last_edit_date instead."""
         _warn_deprecated("last_edition_date", suggest="last_edit_date")
         return self.last_edit_date
 
@@ -145,10 +194,14 @@ class DataNode(_Entity):
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
     def expiration_date(self) -> datetime:
-        if not self._last_edit_date:
-            raise NoData(f"Data node {self.id} has not been written yet.")
+        """Datetime instant of the expiration date of the data node."""
+        last_edit_date = self.last_edit_date
+        validity_period = self._validity_period
 
-        return self._last_edit_date + self.validity_period if self.validity_period else self._last_edit_date
+        if not last_edit_date:
+            raise NoData(f"Data node {self.id} from config {self.config_id} has not been written yet.")
+
+        return last_edit_date + validity_period if validity_period else last_edit_date
 
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
@@ -161,14 +214,18 @@ class DataNode(_Entity):
         self._name = val
 
     @property  # type: ignore
-    @_self_reload(_MANAGER_NAME)
+    def version(self):
+        return self._version
+
+    @property  # type: ignore
     def cacheable(self):
-        return self._cacheable
+        """Deprecated. Use `skippable` attribute of a `Task^` instead."""
+        _warn_deprecated("cacheable", suggest="the skippable feature")
+        return self.properties.get("cacheable", False)
 
     @cacheable.setter  # type: ignore
-    @_self_setter(_MANAGER_NAME)
     def cacheable(self, val):
-        self._cacheable = val
+        _warn_deprecated("cacheable", suggest="the skippable feature")
 
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
@@ -182,29 +239,25 @@ class DataNode(_Entity):
 
     @property  # type: ignore
     def edition_in_progress(self):
-        """
-        Deprecated. Use edit_in_progress instead.
-        """
+        """Deprecated. Use edit_in_progress instead."""
         _warn_deprecated("edition_in_progress", suggest="edit_in_progress")
         return self.edit_in_progress
 
     @edition_in_progress.setter  # type: ignore
     def edition_in_progress(self, val):
+        """Deprecated. Use edit_in_progress instead."""
         _warn_deprecated("edition_in_progress", suggest="edit_in_progress")
         self.edit_in_progress = val
 
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
     def job_ids(self):
-        return self._job_ids
-
-    @job_ids.setter  # type: ignore
-    @_self_setter(_MANAGER_NAME)
-    def job_ids(self, val):
-        self._job_ids = _ListAttributes(self, val)
+        """List of the jobs having edited the data node."""
+        return [edit.get("job_id") for edit in self.edits if edit.get("job_id")]
 
     @property  # type: ignore
     def properties(self):
+        """Dictionary of custom properties."""
         r = _reload(self._MANAGER_NAME, self)
         self._properties = r._properties
         return self._properties
@@ -230,53 +283,71 @@ class DataNode(_Entity):
             return self._properties[protected_attribute_name]
         raise AttributeError(f"{attribute_name} is not an attribute of data node {self.id}")
 
+    def __get_last_modified_datetime(self) -> Optional[datetime]:
+        path = self._properties.get(self.__PATH_KEY, None)
+        if path and os.path.exists(path):
+            return datetime.fromtimestamp(os.path.getmtime(path))
+        return None
+
     @classmethod
     @abstractmethod
     def storage_type(cls) -> str:
-        return NotImplemented
+        return NotImplementedError  # type: ignore
 
     def read_or_raise(self) -> Any:
         """Read the data referenced by this data node.
 
         Returns:
-            Any: The data referenced by this data node.
+            The data referenced by this data node.
         Raises:
             NoData^: If the data has not been written yet.
         """
         if not self.last_edit_date:
-            raise NoData(f"Data node {self.id} has not been written yet.")
+            raise NoData(f"Data node {self.id} from config {self.config_id} has not been written yet.")
         return self._read()
 
     def read(self) -> Any:
         """Read the data referenced by this data node.
 
         Returns:
-            Any: The data referenced by this data node. None if the data has not been written yet.
+            The data referenced by this data node. None if the data has not been written yet.
         """
         try:
             return self.read_or_raise()
         except NoData:
-            self.__logger.warning(f"Data node {self.id} is being read but has never been written.")
+            self.__logger.warning(
+                f"Data node {self.id} from config {self.config_id} is being read but has never been " f"written."
+            )
             return None
 
-    def write(self, data, job_id: Optional[JobId] = None):
+    def write(self, data, job_id: Optional[JobId] = None, **kwargs):
         """Write some data to this data node.
 
         Parameters:
             data (Any): The data to write to this data node.
             job_id (JobId^): An optional identifier of the writer.
+            **kwargs: Extra information to attach to the edit document corresponding to this write.
         """
         from ._data_manager_factory import _DataManagerFactory
 
         self._write(data)
-        self.last_edit_date = datetime.now()  # type: ignore
-        if job_id:
-            self._job_ids.append(job_id)
+        self._track_edit(job_id=job_id, **kwargs)
         self.unlock_edit()
         _DataManagerFactory._build_manager()._set(self)
 
+    def _track_edit(self, **options):
+        """Add Edit tracking information to this data node."""
+        edit = {}
+        for k, v in options.items():
+            if v is not None:
+                edit[k] = v
+        if "timestamp" not in edit:
+            edit["timestamp"] = datetime.now()
+        self.last_edit_date = edit.get("timestamp")
+        self._edits.append(edit)
+
     def lock_edit(self):
-        """Lock the edit of this data node.
+        """Lock the data node modification.
 
         Note:
             The data node can be unlocked with the method `(DataNode.)unlock_edit()^`.
@@ -284,27 +355,23 @@ class DataNode(_Entity):
         self.edit_in_progress = True
 
     def lock_edition(self):
-        """
-        Deprecated. Use lock_edit instead.
-        """
+        """Deprecated. Use lock_edit instead."""
         _warn_deprecated("lock_edition", suggest="lock_edit")
         self.lock_edit()
 
     def unlock_edit(self, at: datetime = None, job_id: JobId = None):
-        """Unlocks the edit of the data node.
+        """Unlocks the data node modification.
 
         Parameters:
             at (datetime): Deprecated.
             job_id (JobId^): Deprecated.
         Note:
-            The data node  can be locked with the method `(DataNode.)lock_edit()^`.
+            The data node can be locked with the method `(DataNode.)lock_edit()^`.
         """
         self.edit_in_progress = False  # type: ignore
 
     def unlock_edition(self, at: datetime = None, job_id: JobId = None):
-        """
-        Deprecated. Use (DataNode.)unlock_edit()^` instead.
-        """
+        """Deprecated. Use `(DataNode.)unlock_edit()^` instead."""
         _warn_deprecated("unlock_edition", suggest="unlock_edit")
         self.unlock_edit()
 
@@ -325,32 +392,36 @@ class DataNode(_Entity):
         if len(operators) == 0:
             return data
         if not ((type(operators[0]) == list) or (type(operators[0]) == tuple)):
-            if isinstance(data, pd.DataFrame):
+            if isinstance(data, (pd.DataFrame, modin_pd.DataFrame)):
                 return DataNode.__filter_dataframe_per_key_value(data, operators[0], operators[1], operators[2])
             if isinstance(data, List):
                 return DataNode.__filter_list_per_key_value(data, operators[0], operators[1], operators[2])
         else:
-            if isinstance(data, pd.DataFrame):
+            if isinstance(data, (pd.DataFrame, modin_pd.DataFrame)):
                 return DataNode.__filter_dataframe(data, operators, join_operator=join_operator)
             if isinstance(data, List):
                 return DataNode.__filter_list(data, operators, join_operator=join_operator)
-        return NotImplemented
+        return NotImplementedError
 
     @staticmethod
-    def __filter_dataframe(df_data: pd.DataFrame, operators: Union[List, Tuple], join_operator=JoinOperator.AND):
+    def __filter_dataframe(
+        df_data: Union[pd.DataFrame, modin_pd.DataFrame], operators: Union[List, Tuple], join_operator=JoinOperator.AND
+    ):
         filtered_df_data = []
         if join_operator == JoinOperator.AND:
             how = "inner"
         elif join_operator == JoinOperator.OR:
             how = "outer"
         else:
-            return NotImplemented
+            return NotImplementedError
         for key, value, operator in operators:
             filtered_df_data.append(DataNode.__filter_dataframe_per_key_value(df_data, key, value, operator))
         return DataNode.__dataframe_merge(filtered_df_data, how) if filtered_df_data else pd.DataFrame()
 
     @staticmethod
-    def __filter_dataframe_per_key_value(df_data: pd.DataFrame, key: str, value, operator: Operator):
+    def __filter_dataframe_per_key_value(
+        df_data: Union[pd.DataFrame, modin_pd.DataFrame], key: str, value, operator: Operator
+    ):
         df_by_col = df_data[key]
         if operator == Operator.EQUAL:
             df_by_col = df_by_col == value
@@ -382,7 +453,7 @@ class DataNode(_Entity):
         elif join_operator == JoinOperator.OR:
             return list(set(np.concatenate(filtered_list_data)))
         else:
-            return NotImplemented
+            return NotImplementedError
 
     @staticmethod
     def __filter_list_per_key_value(list_data: List, key: str, value, operator: Operator):
@@ -409,11 +480,11 @@ class DataNode(_Entity):
 
     @abstractmethod
     def _read(self):
-        return NotImplemented
+        raise NotImplementedError
 
     @abstractmethod
     def _write(self, data):
-        return NotImplemented
+        raise NotImplementedError
 
     def __getitem__(self, items):
         return _FilterDataNode(self.id, self._read())[items]
@@ -424,7 +495,7 @@ class DataNode(_Entity):
         """Indicate if this data node is ready for reading.
 
         Returns:
-            bool: False if the data is locked for modification or if the data has never been written.
+            False if the data is locked for modification or if the data has never been written.
                 True otherwise.
         """
         if self._edit_in_progress:
@@ -436,14 +507,18 @@ class DataNode(_Entity):
 
     @property  # type: ignore
     @_self_reload(_MANAGER_NAME)
-    def _is_in_cache(self):
-        if not self._cacheable:
-            return False
+    def is_up_to_date(self):
+        """Indicate if this data node is up-to-date.
+
+        Returns:
+            False if the data ever been written or the expiration date has passed.
+                True otherwise.
+        """
         if not self._last_edit_date:
             # Never been written so it is not up-to-date
             return False
         if not self._validity_period:
-            # No validity period and cacheable so it is up-to-date
+            # No validity period and has already been written, so it is up-to-date
             return True
         if datetime.now() > self.expiration_date:
             # expiration_date has been passed
